# !/usr/bin/env python3
""" base api """
from typing import Optional
import wenxin_api
import wenxin_api.variable
from wenxin_api import requestor, error
from wenxin_api.base_object import BaseObject
from wenxin_api.const import ERNIE_1P5B_MODEL
from wenxin_api.error import IllegalRequestArgumentError, InvalidResponseValue

class APIBaseObject(BaseObject):
    wenxin_api.variable.API_REQUEST_URLS

    def __init__(self, 
                 ak=None, 
                 sk=None, 
                 api_type=None, 
                 base_model=ERNIE_1P5B_MODEL,
                 **params):
        super(APIBaseObject, self).__init__(**params)
        self.__setattr__("ak", ak)
        self.__setattr__("sk", sk)
        self.__setattr__("api_type", api_type)
        self.__setattr__("base_model", base_model)

    @classmethod
    def construct_from(cls, **values):
        instance = cls(**values)
        return instance

    @classmethod
    def default_request(
        cls,
        ak=None,
        sk=None,
        api_type=None,
        method="post",
        request_id=None,
        request_type=None,
        files=None,
        **params,
    ):
        if api_type is None:
            api_type = cls.api_type
        try:
            api_type = int(api_type)
            url = wenxin_api.variable.API_REQUEST_URLS[api_type]
        except:
            raise IllegalRequestArgumentError()
        if request_type is None:
            request_type = params.pop("type", None)

        http_requestor = requestor.HTTPRequestor(ak, sk, request_type)
        resp = http_requestor.request(url, method, files=files, request_id=request_id, **params)
        if not isinstance(resp, BaseObject) and isinstance(resp, list):
            InvalidResponseValue()
        return resp
    
    @classmethod
    def get_url(cls, api_type=None):
        try:
            if api_type is None:
                api_type = cls.api_type
            url = wenxin_api.variable.API_REQUEST_URLS[api_type]
        except:
            raise IllegalRequestArgumentError("api_type: {}".format(api_type))
        return url

class CreatableAPIObject(APIBaseObject):
    """ creatable api object """
    @classmethod
    def create(cls, ak=None, sk=None, api_type=None, request_id=None, **params):
        """ create """
        if isinstance(cls, APIBaseObject):
            raise ValueError(".create may only be called as a class method now.")
        request_type = params.pop("type", None)
        method = params.pop("method", "post")
        http_requestor = requestor.HTTPRequestor(ak, sk, request_type)
        url = cls.get_url(api_type)
        resp = http_requestor.request(url, method, request_id=request_id, **params)
        if not isinstance(resp, BaseObject):
            raise InvalidResponseValue("create method should returns BaseObject instance")
        return cls.construct_from(**resp, api_type=api_type)

class DeletableAPIObject(APIBaseObject):
    """ deletable api object """
    @classmethod
    def delete(cls, ak=None, sk=None, api_type=None, request_id=None, **params):
        if isinstance(cls, APIBaseObject):
            raise ValueError(".delete may only be called as a class method now.")
        request_type = params.pop("type", None)
        method = params.pop("method", "post")
        http_requestor = requestor.HTTPRequestor(ak, sk, request_type)
        url = cls.get_url(api_type)
        resp = http_requestor.request(url, method, request_id=request_id, **params)
        if not isinstance(resp, BaseObject):
            raise InvalidResponseValue("delete method should returns BaseObject instance")
        return cls.construct_from(api_type=api_type, **resp)

class StopableAPIObject(APIBaseObject):
    """ stopable api object """
    @classmethod
    def stop(cls, ak=None, sk=None, api_type=None, request_id=None, **params):
        request_type = params.pop("type", None)
        method = params.pop("method", "post")
        http_requestor = requestor.HTTPRequestor(ak, sk, request_type)
        url = cls.get_url(api_type)
        resp = http_requestor.request(url, method, request_id=request_id, **params)
        if not isinstance(resp, BaseObject):
            raise InvalidResponseValue("stop method should returns BaseObject instance")
        return cls.construct_from(api_type=api_type, **resp)

class ListableAPIObject(APIBaseObject):
    """ listable api object """
    @classmethod
    def list(cls, ak=None, sk=None, api_type=None, request_id=None, **params):
        request_type = params.pop("type", None)
        method = params.pop("method", "post")
        if request_type is None:
            raise IllegalRequestArgumentError("type is not provided")
        http_requestor = requestor.HTTPRequestor(ak, sk, request_type)
        url = cls.get_url(api_type)
        resps = http_requestor.request(url, method, request_id=request_id, **params)
        if isinstance(resps, BaseObject):
            return [cls.construct_from(**resps, api_type=api_type)]
        resp = [cls.construct_from(**r, api_type=api_type) for r in resps]
        return resp

class RetrievalableAPIObject(APIBaseObject):
    """ retrievalable api object """
    @classmethod
    def retrieve(cls, ak=None, sk=None, api_type=None, request_id=None, **params):
        if "type" not in params:
            raise IllegalRequestArgumentError("type is not provided")
        request_type = params.pop("type", None)
        http_requestor = requestor.HTTPRequestor(ak, sk, request_type)
        method = params.pop("method", "post")
        url = cls.get_url(api_type)
        resp = http_requestor.request(url, method, request_id=request_id, **params)
        if not isinstance(resp, BaseObject):
            raise InvalidResponseValue("retrieve method should returns BaseObject instance")
        return cls.construct_from(api_type=api_type, **resp)


# !/usr/bin/env python3
""" base object """

class BaseObject(dict):
    """ base object """
    def __init__(self, **params):
        super(BaseObject, self).__init__()
        for k, v in params.items():
            self.__setattr__(k, v)

    def __setattr__(self, k, v):
        if k[0] == "_" or k in self.__dict__:
            super(BaseObject, self).__setattr__(k, v)
        else:
            self[k] = v

    def __getattr__(self, k):
        if k[0] == "_":
            raise AttributeError(k)
        try:
            return self[k]
        except KeyError as err:
            raise AttributeError(*err.args)

    @property
    def id(self):
        return self.get(f"{self.type}_id", "")

    @property
    def status(self):
        return self.get(f"{self.type}_state", None)

    def refresh_from(self, new_obj):
        for k, v in new_obj.items():
            self.__setattr__(k, v)

    @classmethod
    def construct_from(cls, **values):
        instance = cls(**values)
        return instance


""" dataset api """
import json
import os
import time

import wenxin_api
from wenxin_api import requestor, error, log
from wenxin_api.api import CreatableAPIObject, ListableAPIObject, DeletableAPIObject, RetrievalableAPIObject
from wenxin_api.error import MissingRequestArgumentError, FileError
from json import JSONDecodeError
from wenxin_api.const import CMD_UPLOAD_DATA, CMD_QUERY_DATA, CMD_DELETE_DATA, FILE_LINE_UPPER_BOUND
logger = log.get_logger()


class Dataset(CreatableAPIObject, ListableAPIObject, DeletableAPIObject, RetrievalableAPIObject):
    """ dataset api """
    training_api_type = 10

    @staticmethod
    def _check_file_line(file_path):
        with open(file_path, 'r') as f:
            for line in f:
                try:
                    json_line = json.loads(line)
                    src = json_line.get("src", None)
                    tgt = json_line.get("tgt", None)
                    if src is None or tgt is None:
                        raise FileError("missing src or tgt column")
                    if len(src) > FILE_LINE_UPPER_BOUND or len(tgt) > FILE_LINE_UPPER_BOUND:
                        raise FileError("the file line exceeded the maximum word limit!")
                except JSONDecodeError as e:
                    raise FileError("input file must be in json format")


    @classmethod
    def create(cls, local_file_path, **params):
        """ create """
        api_type = params.pop("api_type", cls.training_api_type)
        need_check = params.pop("need_check", True)
        if need_check:
            cls._check_file_line(local_file_path)
        timeout = params.pop("timeout", None)
        if "data_name" not in params:
            params["data_name"] = "test"
        params["type"] = "data"

        with open(local_file_path, 'rb') as f:
            files = {"url": ("test", f)}
            request_id = CMD_UPLOAD_DATA        
            headers = {}
            resp = cls.default_request(headers=headers,
                                    request_id=request_id, 
                                    api_type=api_type,
                                    files=files,
                                    **params)
        dataset = cls.retrieve(data_id=resp.id, api_type=api_type)
        return dataset

    @classmethod
    def retrieve(cls, *args, **params):
        """ retrieve """
        api_type = params.pop("api_type", cls.training_api_type)
        request_id = CMD_QUERY_DATA
        params["type"] = "data"
        if "data_id" not in params:
            raise MissingRequestArgumentError("data_id is not provided")
        resp = super().retrieve(request_id=request_id, api_type=api_type, **params)
        return resp

    @classmethod
    def list(cls, *args, **params):
        """ list """
        api_type = params.pop("api_type", cls.training_api_type)
        request_id = CMD_QUERY_DATA
        params["type"] = "data"
        resp = super().list(request_id=request_id, api_type=api_type, **params)
        return resp

    @classmethod
    def delete(cls, *args, **params):
        """ delete """
        api_type = params.pop("api_type", cls.training_api_type)
        request_id = CMD_DELETE_DATA
        params["type"] = "data"
        if "data_id" not in params:
            raise MissingRequestArgumentError("data_id is not provided")
        return super().delete(request_id=request_id, api_type=api_type, **params)

    def __str__(self):
        return "Dataset {}:{}".format(
                        id(self),
                        json.dumps({"id": self.id, 
                                    "name": self.get("data_name", ""),
                                    "api_type": self.api_type,
                                    "url": self.get("url", ""),
                                    "md5": self.get("md5", ""),
                                    "type": self.type
                                   }, ensure_ascii=False)
        )

    def __repr__(self):
        return self.__str__()



# !/usr/bin/env python3
""" train api """
import json
from wenxin_api import requestor, error
from wenxin_api.api import (APIBaseObject, 
                            CreatableAPIObject, 
                            ListableAPIObject, 
                            StopableAPIObject, 
                            RetrievalableAPIObject)
from wenxin_api.base_object import BaseObject
from wenxin_api.const import CMD_DO_TRAIN, CMD_QUERY_TASK, CMD_STOP_TASK
from wenxin_api.const import TRAIN_TASK
from wenxin_api.error import MissingRequestArgumentError, IllegalRequestArgumentError

class Train(ListableAPIObject, CreatableAPIObject, StopableAPIObject, RetrievalableAPIObject):
    OBJECT_NAME = "fine-tunes"
    training_api_type = 10

    @classmethod
    def create(cls, train_datasets=[], dev_datasets=[], **params):
        if not isinstance(train_datasets, list):
            raise IllegalRequestArgumentError("train_datasets should be a list")
        if not isinstance(dev_datasets, list):
            raise IllegalRequestArgumentError("dev_datasets should be a list")
        if len(train_datasets) == 0:
            raise IllegalRequestArgumentError("train datasets shouldn't be null")

        train_data_ids = [dataset.id for dataset in train_datasets]
        if len(dev_datasets) > 0:
            dev_data_ids = [dataset.id for dataset in dev_datasets]
        else:
            dev_data_ids = []

        request_id = CMD_DO_TRAIN
        params["type"] = "task"
        # api_type 10 for training
        api_type = params.pop("api_type", cls.training_api_type)
        resp = super().create(request_id=request_id, 
                              api_type=api_type,
                              train_data_ids=train_data_ids, 
                              dev_data_ids=dev_data_ids, 
                              **params)
        resp.update()
        return resp

    @classmethod
    def list(cls, *args, **params):
        """ list """
        request_id = CMD_QUERY_TASK
        api_type = params.pop("api_type", cls.training_api_type)
        params["type"] = "task"
        resps = super().list(request_id=request_id, 
                             api_type=api_type,
                             **params)
        filtered_resps = [resp for resp in resps \
                            if resp.status >= 200 and \
                               resp.status < 300]
        return filtered_resps

    @classmethod
    def retrieve(cls, *args, **params):
        """ retrieve """
        request_id = CMD_QUERY_TASK
        api_type = params.pop("api_type", cls.training_api_type)
        params["type"] = "task"
        if "task_id" not in params:
            raise MissingRequestArgumentError("task_id is not provided")
        resp = super().retrieve(request_id=request_id, 
                                api_type=api_type,
                                **params)
        return resp

    def stop(self, *args, **params) -> BaseObject:
        request_id = CMD_STOP_TASK
        params["type"] = "task"
        params["task_type"] = TRAIN_TASK
        params["task_id"] = self.id
        if self.id is None:
            raise IllegalRequestArgumentError("illegal task_id")
        super().stop(request_id=request_id, 
                     api_type=self.api_type,
                     **params)
        self.update()

    def update(self, *args, **params):
        """ update """
        request_id = CMD_QUERY_TASK
        params["type"] = "task"
        task = super().retrieve(task_id=self.id, 
                                request_id=request_id, 
                                api_type=self.api_type,
                                **params)
        self.refresh_from(task)

    def __str__(self):
        return "Task {}:{}".format(
                        id(self),
                        json.dumps({"id": self.id, 
                                    "status": self.status,
                                    "api_type": self.api_type,
                                    "base_model": self.base_model,
                                    "job_id": self.job_id,
                                    "type": self.type
                                   }, ensure_ascii=False)
        )

    def __repr__(self):
        return self.__str__()




""" text generation task """
import time
import warnings
import json
import wenxin_api
from wenxin_api import requestor, error, log
from wenxin_api.api import Task
from wenxin_api.api import ListableAPIObject
import wenxin_api.variable
from wenxin_api.const import SOURCE_WENXIN, SOURCE_CONSOLE
from wenxin_api.error import APIError
logger = log.get_logger()

class TextToImage(Task):
    """ text generation task """
    OBJECT_NAME = "text_to_image"

    def _http_init(self):
        """ http initialization """
        self.requestor = requestor.HTTPRequestor()
        self.create_url = wenxin_api.variable.VILG_CREATE_URL
        self.retrieve_url = wenxin_api.variable.VILG_RETRIEVE_URL

    @classmethod
    def create(cls, *args, **params):
        """ create """
        cls._http_init(cls)            
        start = time.time()
        timeout = params.pop("timeout", None)
        text = params.pop("text", "")
        style = params.pop("style", "")
        watermark = params.get("watermark", 1)
        num = params.get("num", 6)
        resolution = params.get("resolution", "1024*1024")
        resp = cls.requestor.request(cls.create_url, 
                                      text=text, 
                                      style=style, 
                                      watermark=watermark,
                                      num=num,
                                      resolution=resolution,
                                      return_raw=True)

        try:
            task_id = resp.json()["data"]["taskId"]
        except Exception as e:
            raise APIError(json.dumps(resp.json(), 
                           ensure_ascii=False,
                           indent=2))
        not_ready = True
        while not_ready:
            resp = cls.requestor.request(cls.retrieve_url, taskId=task_id, return_raw=True)
            not_ready = resp.json()["data"]["status"] == 0
            if not not_ready:
                return cls._resolve_result(resp.json())
            rst = resp.json()
            logger.info("model is painting now!, taskId: {}, waiting: {}".format(
                rst["data"]["taskId"],
                rst["data"]["waiting"]))
            time.sleep(wenxin_api.variable.REQUEST_SLEEP_TIME)

    @staticmethod
    def _resolve_result(resp):
        if resp["code"] == 0:
            ret_dict = {"imgUrls": []}
            for d in resp["data"]["imgUrls"]:
                ret_dict["imgUrls"].append(d["image"])
            return ret_dict
        else:
            return resp


class TextToImageRef(TextToImage):
    """ text generation task """
    OBJECT_NAME = "text_to_image"

    @classmethod
    def create(cls, *args, **params):
        """ create """
        cls._http_init(cls)   
        print("create url", cls.create_url)         
        start = time.time()
        timeout = params.pop("timeout", None)
        text = params.pop("text", "")
        style = params.pop("style", "")
        watermark = params.get("watermark", 1)
        num = params.get("num", 6)
        resolution = params.get("resolution", "1024*1024")
        local_file_path = params.get("local_file_path", None)


        with open(local_file_path, 'rb') as f:
            files = {"url": ("test", f)}
            headers = {}
            # request_id = CMD_UPLOAD_DATA        
            # resp = cls.default_request(headers=headers,
            #                         request_id=request_id, 
            #                         files=files,
            #                         **params)
        
            resp = cls.requestor.request(cls.create_url, 
                                        headers=headers,
                                        files=files,
                                        text=text, 
                                        style=style, 
                                        watermark=watermark,
                                        num=num,
                                        resolution=resolution,
                                        return_raw=True)

        try:
            task_id = resp.json()["data"]["taskId"]
        except Exception as e:
            raise APIError(json.dumps(resp.json(), 
                           ensure_ascii=False,
                           indent=2))
        not_ready = True
        while not_ready:
            resp = cls.requestor.request(cls.retrieve_url, taskId=task_id, return_raw=True)
            not_ready = resp.json()["data"]["status"] == 0
            if not not_ready:
                return cls._resolve_result(resp.json())
            rst = resp.json()
            logger.info("model is painting now!, taskId: {}, waiting: {}".format(
                rst["data"]["taskId"],
                rst["data"]["waiting"]))
            time.sleep(wenxin_api.variable.REQUEST_SLEEP_TIME)

    @staticmethod
    def _resolve_result(resp):
        if resp["code"] == 0:
            ret_dict = {"imgUrls": []}
            for d in resp["data"]["imgUrls"]:
                ret_dict["imgUrls"].append(d["image"])
            return ret_dict
        else:
            return resp


class TextToImageConsole(TextToImage):
    """ text generation task """
    OBJECT_NAME = "text_to_image_console"

    def _http_init(self):
        """ http initialization """
        self.requestor = requestor.ConsoleHTTPRequestor()
        access_token = self.requestor._get_access_token()
        self.create_url = wenxin_api.variable.VILG_CREATE_URL_CONSOLE + "?access_token={}".format(access_token)
        self.retrieve_url = wenxin_api.variable.VILG_RETRIEVE_URL_CONSOLE + "?access_token={}".format(access_token)

    @staticmethod
    def _resolve_result(resp):
        ret_dict = {"imgUrls": []}
        for d in resp["data"]["imgUrls"]:
            ret_dict["imgUrls"].append(d["image"])
        return ret_dict


""" text generation task """
import time
import warnings
import json
import wenxin_api
from wenxin_api import requestor, error, log
from wenxin_api.api import Task, Train
from wenxin_api.api import CreatableAPIObject, ListableAPIObject
import wenxin_api.variable
from wenxin_api.const import SOURCE_WENXIN, SOURCE_CONSOLE
from wenxin_api.error import APIError, IllegalRequestArgumentError
from wenxin_api.const import (CMD_DO_DEPLOY, 
                              CMD_DO_INFERENCE, 
                              CMD_QUERY_MODEL, 
                              CMD_QUERY_TASK,
                              CMD_STOP_TASK,)
from wenxin_api.const import (MODEL_STATE_ERROR, 
                              MODEL_STATE_SUCCESS, 
                              MODEL_STATE_ON_SERVICE,)
from wenxin_api.const import (TASK_STATE_INFER_IN_QUEUE,
                              TASK_STATE_INFER_RUNNING,
                              TASK_STATE_INFER_END,)
from wenxin_api.const import (TASK_STATE_DEPLOY_SUBMIT, 
                              TASK_STATE_DEPLOY_IN_QUEUE, 
                              TASK_STATE_DEPLOY_RUNNING,
                              TASK_STATE_DEPLOY_SUCCESS,
                              TASK_STATE_DEPLOY_FAILED,)
logger = log.get_logger()

class TextToImageTuned(Task):
    """ text generation task """
    OBJECT_NAME = "text_to_image"
    training_api_type = 12

    def _http_init(self):
        """ http initialization """
        self.requestor = requestor.HTTPRequestor()

    @classmethod
    def _create(cls, *args, **params):
        """ create """
        cls._http_init(cls)            
        start = time.time()
        api_type = params.pop("api_type", None)
        timeout = params.pop("timeout", None)
        text = params.pop("text", "")
        style = params.pop("style", "")
        num = params.get("num", 6)
        resolution = params.get("resolution", "1024*1024")
        model = params.get("model", None)
        
        request_id = params.get("request_id", None)
        resp = cls.requestor.request(cls.get_url(api_type), 
                                      model_id=model.id,
                                      text=text, 
                                      style=style, 
                                      resolution=resolution,
                                      request_id=request_id,
                                      return_raw=True)

        try:
            task_id = resp.json()["data"]["taskId"]
        except Exception as e:
            raise APIError(json.dumps(resp.json(), 
                           ensure_ascii=False,
                           indent=2))
        not_ready = True
        while not_ready:
            resp = cls.requestor.request(wenxin_api.variable.VILG_TUNING_QUERY_URL, 
                                         request_id=request_id,
                                         model_id=model.id,
                                         taskId=task_id, 
                                         return_raw=True)
            print("resp:", resp.json())
            not_ready = resp.json()["data"]["status"] == 0
            if not not_ready:
                return cls._resolve_result(resp.json())
            rst = resp.json()
            logger.info("model is painting now!, taskId: {}, waiting: {}".format(
                rst["data"]["taskId"],
                rst["data"]["waiting"]))
            time.sleep(wenxin_api.variable.REQUEST_SLEEP_TIME)

    @staticmethod
    def _resolve_result(resp):
        if resp["code"] == 0:
            ret_dict = {"imgUrls": []}
            for d in resp["data"]["imgUrls"]:
                ret_dict["imgUrls"].append(d["image"])
            return ret_dict
        else:
            return resp

    @classmethod
    def create(cls, text=None, model=None, **params):
        """ create """
        api_type = params.pop("api_type", cls.training_api_type)
        if text is None:
            raise IllegalRequestArgumentError("text shouldn't be none")
        if model is None:
            raise IllegalRequestArgumentError("model shouldn't be none")
        else:
            model.update()
        # deploy
        if model.status != MODEL_STATE_ON_SERVICE:
            request_id = CMD_DO_DEPLOY
            params["type"] = "task"
            deploy_task = CreatableAPIObject.create(model_id=model.id, 
                                                    request_id=request_id, 
                                                    api_type=api_type,
                                                    **params)

            not_ready = True
            while not_ready:
                deploy_task.update()
                print("deploy task:", deploy_task)
                not_ready = deploy_task.status != TASK_STATE_DEPLOY_SUCCESS
                if deploy_task.status == TASK_STATE_DEPLOY_FAILED:
                    raise APIError("deploy task:{} failed".format(deploy_task.id))
                logger.info("model is preparing now!, task_id:{}, status:{}".format(
                    deploy_task.id, 
                    deploy_task.status)
                )
                time.sleep(wenxin_api.variable.REQUEST_SLEEP_TIME)
        
        # inference
        api_type = params.pop("api_type", cls.training_api_type)
        logger.info("model {}: starts writing".format(model.id))
        request_id = CMD_DO_INFERENCE
        # todo:add base_model and is_prompt
        resp = cls._create(text=text, 
                           model=model, 
                           api_type=api_type,
                           request_id=request_id, 
                           **params)

        return resp



# !/usr/bin/env python3
""" free qa task """
import time
import json
import wenxin_api
from wenxin_api.api import Task
from wenxin_api.error import IllegalRequestArgumentError
from wenxin_api import requestor,log
from wenxin_api.error import APIError
logger = log.get_logger()

class FreeQA(Task):
    """ free qa task """
    @staticmethod
    def _resolve_result(resp):
        rst = {}
        rst["result"] = resp["result"]
        return rst
    
    def _http_init(self):
        """ http initialization """
        self.requestor = requestor.HTTPRequestor()
        self.api_type = 5
        self.create_url = wenxin_api.variable.API_REQUEST_URLS[self.api_type]
        self.retrieve_url = wenxin_api.variable.API_GET_RESULT_URL

        


    @classmethod
    def create(cls, text=None, model=None, **params):
        """ create """
        cls._http_init(cls) 

        if text is None:
            raise IllegalRequestArgumentError("text shouldn't be none")
        model_id = 1 if model is None else model.id
        logger.info("model {}: starts writing".format(model_id))
        
        is_async = params.pop("async", 1)
        params["async"] = is_async
        resp = cls.requestor.request(cls.create_url, "post", text=text, **params, return_raw=True)
        
        try:
            task_id = resp.json()["data"]["taskId"]
        except Exception as e:
            raise APIError(json.dumps(resp.json(), 
                           ensure_ascii=False,
                           indent=2))

        not_ready = True
        while not_ready:
            resp = cls.requestor.request(cls.retrieve_url, taskId=task_id, return_raw=True)
            not_ready = resp.json()["data"]["status"] == 0
            if not not_ready:
                return cls._resolve_result(resp.json()["data"])
            rst = resp.json()
            logger.info("model is running now!, taskId: {}".format(rst["data"]["taskId"]))
            time.sleep(wenxin_api.variable.REQUEST_SLEEP_TIME)




# -*- coding: utf-8 -*
""" wenxin global const """

# command id
CMD_BASE_START = 1000

CMD_DO_TRAIN = CMD_BASE_START + 1  # start a train task
CMD_DO_DEPLOY = CMD_BASE_START + 2  # start a deployment task
CMD_DO_INFERENCE = CMD_BASE_START + 3  # start an inference task

CMD_QUERY_TASK = CMD_BASE_START + 5  # get task status
CMD_STOP_TASK = CMD_BASE_START + 6  # stop a task

CMD_QUERY_MODEL = CMD_BASE_START + 9  # get model status
CMD_DELETE_MODEL = CMD_BASE_START + 11  # delete a model

CMD_UPLOAD_DATA = CMD_BASE_START + 12  # upload a dataset to bos
CMD_DELETE_DATA = CMD_BASE_START + 14  # delete a dataset
CMD_QUERY_DATA = CMD_BASE_START + 15  # get dataset status
CMD_STOP_SERVER = CMD_BASE_START + 16 # kill a deploy server

# task type
TRAIN_TASK=1
INFERENCE_TASK=2

# error code in general, from 4000 to 4999
ERR_CODE_BASE = 4000

ERR_CODE_REQUEST_LOAD = ERR_CODE_BASE + 1
ERR_MSG_REQUEST_LOAD = "request load fail, it must be json."
ERR_CODE_PARAMS_PARSER = ERR_CODE_BASE + 2
ERR_MSG_PARAMS_PARSER = "params is not needed, please check."
ERR_CODE_DATA_FORMAT = ERR_CODE_BASE + 3
ERR_MSG_DATA_FORMAT = "invalid data format, plase check."
ERR_CODE_JSON_FORMAT = ERR_CODE_BASE + 4
ERR_MSG_JSON_FORMAT = "dumps to json format string failed"
ERR_CODE_LOAD_DATA = ERR_CODE_BASE + 5
ERR_MSG_LOAD_DATA = "error in upload file, please check."
# tba

# erro code in training module, from 5000 to 5999
ERR_CODE_BASE_TRAIN = 5000

ERR_CODE_SUBMIT_PDC = ERR_CODE_BASE_TRAIN + 1
ERR_MSG_SUBMIT_PDC = "error when submitting to paddlecloud platform"
# tba


# erro code in deployment module, from 6000 to 6999
ERR_CODE_BASE_INFER = 6000

ERR_CODE_SUBMIT_XVISION = ERR_CODE_BASE_INFER + 1
ERR_MSG_SUBMIT_XVISION = "error when submitting to xvision platform"
ERR_CODE_WENXIN_INFER = ERR_CODE_BASE_INFER + 2
ERR_MSG_WENXIN_INFER = "internal error in model"
# tba


# task type
TASK_TYPE_COMMON = 0
TASK_TYPE_TRAIN = 1
TASK_TYPE_DEPLOY = 2
TASK_TYPE_INFER = 3

# base model
ERNIE_100B_MODEL = 1
ERNIE_10B_MODEL = 2
ERNIE_1P5B_MODEL = 3

# model status
MODEL_STATE_ERROR = 0
MODEL_STATE_SUCCESS = 1
MODEL_STATE_ON_SERVICE = 2

# task status
# create task, from 100 to 199
TASK_STATE_CREATE_BASE = 100
TASK_STATE_CREATE_SUCCESS = TASK_STATE_CREATE_BASE + 1
TASK_STATE_CREATE_FAILED = TASK_STATE_CREATE_BASE + 2

# training task, from 200 to 299
TASK_STATE_TRAIN_BASE = 200
TASK_STATE_TRAIN_SUBMIT = TASK_STATE_TRAIN_BASE + 1
TASK_STATE_TRAIN_IN_QUEUE = TASK_STATE_TRAIN_BASE + 2
TASK_STATE_TRAIN_RUNNING = TASK_STATE_TRAIN_BASE + 3
TASK_STATE_TRAIN_SUCCESS = TASK_STATE_TRAIN_BASE + 4
TASK_STATE_TRAIN_FAILED = TASK_STATE_TRAIN_BASE + 5
TASK_STATE_TRAIN_KILLED = TASK_STATE_TRAIN_BASE + 6


# deployment task, from 300 to 399
TASK_STATE_DEPLOY_BASE = 300
TASK_STATE_DEPLOY_SUBMIT = TASK_STATE_DEPLOY_BASE + 1
TASK_STATE_DEPLOY_IN_QUEUE = TASK_STATE_DEPLOY_BASE + 2
TASK_STATE_DEPLOY_RUNNING = TASK_STATE_DEPLOY_BASE + 3
TASK_STATE_DEPLOY_SUCCESS = TASK_STATE_DEPLOY_BASE + 4
TASK_STATE_DEPLOY_FAILED = TASK_STATE_DEPLOY_BASE + 5

# inference task, from 400 to 499
TASK_STATE_INFER_BASE = 400
TASK_STATE_INFER_IN_QUEUE = TASK_STATE_INFER_BASE + 1
TASK_STATE_INFER_RUNNING = TASK_STATE_INFER_BASE + 2
TASK_STATE_INFER_END = TASK_STATE_INFER_BASE + 3

# 
TYPE_TEXT_TO_IMAGE = 12

# file
FILE_LINE_UPPER_BOUND = 1000

# source
SOURCE_WENXIN = 1
SOURCE_CONSOLE = 2